<?php 

/**
* Класс для работы с ddsx файлами
* 
*/

class Ddsx { 

	const MAGIC = "DDSx";
	const NOT_COMPRESSED = 0x00;
	const ZSTD_COMPRESSED = 0x20;
	const LZMA_COMPRESSED = 0x40;
	const OODLE_COMPRESSED = 0x60;
	const ZLIB_COMPRESSED = 0x80;

	private static $stream = NULL;

	static public function convertFile( $sourceFile, $ddsfile ) {

		if( self::$stream === NULL ){

			self::$stream = new DataStream();

		}

		if( !self::$stream->open( $sourceFile ) ){

			return ErrorHandler::error( "Не удалось открыть файл '{$sourceFile}'" );			

		}

		return self::convert( $ddsfile );
	}

	static public function convertData( $data, $ddsfile ) {

		if( self::$stream === NULL ){

			self::$stream = new DataStream();

		}

		if( !self::$stream->set( $data ) ){

			return ErrorHandler::error( "Не удалось открыть файл '{$sourceFile}'" );			

		}

		return self::convert( $ddsfile );

	}

	static private function convert( $ddsfile ) {

		// проверка сигнатуры "DDSx""
		$signature = self::$stream->string( 4 );

		if( $signature !== self::MAGIC ) {

			return ErrorHandler::error( "Исходный файл не является файлом ddsx" ); 

		} 

		// заголовок ddsx файла   
		$format = self::$stream->string( 4 );
		$flags[0] = self::$stream->byte();
		$flags[1] = self::$stream->byte();
		$flags[2] = self::$stream->byte();
		$flags[3] = self::$stream->byte();
		$width = self::$stream->uint16();
		$height = self::$stream->uint16();
		$mipmaps = self::$stream->byte();  //????
		$flags[5] = self::$stream->byte();
		$flags[6] = self::$stream->byte();
		$flags[7] = self::$stream->byte();  		
		$mipmaps = 0;
		$bitsperpixel = self::$stream->uint16();
		$unknown3 = self::$stream->uint16();
		$unpacked = self::$stream->uint32();
		$packed = self::$stream->uint32();            

		$header_magic = "DDS ";
		$header_dwSize = 124;
		$header_dwHeight = $height;
		$header_dwWidth = $width;
		$header_dwDepth = 0;
		$header_dwMipMapCount = ($mipmaps == 1) ? 0 : $mipmaps; 
		$header_dwCaps = 0x1000 + (0x8 + 0x400000) * ( $mipmaps > 1 );
		$pixelformat_dwSize = 32;   

		// непонятная херня
		if( ( $flags[2] == 0x0A )
		|| ( $flags[2] == 0x0B ) ) {  
			//		return ErrorHandler::error( "Неизвестная ошибка в файле ddsx" );     
		} 

		switch( $flags[3]  ) {

			case self::NOT_COMPRESSED : 

				$data = self::$stream->get( 0x20 );

				if( !$data ){

					return ErrorHandler::error( "Нет изображения" );	

				}

				break;

			case self::ZSTD_COMPRESSED :

				$data = self::$stream->get( 0x20 );
				$data = Zstd::decode( $data );

				if( !$data ){

					return ErrorHandler::error( "Oшибка распаковки zstd cжатых данных" );	

				}

				break;

			case self::LZMA_COMPRESSED :

				$data = self::$stream->get( 0x20 );
				$data = Lzma::decode( substr( $data, 0, 5 ) . pack( 'Q', $unpacked ) . substr( $data, 5 ) );

				if( !$data ){

					return ErrorHandler::error( "Oшибка распаковки lzma cжатых данных" );	

				}

				break;

			case self::ZLIB_COMPRESSED :

				$data = self::$stream->get( 0x20 );
				$data = zlib_decode( $data );

				if( $data === false ) { 

					return ErrorHandler::error( "Oшибка распаковки zlib cжатых данных" );

				} 

				break; 

			case self::OODLE_COMPRESSED : 

				$data = self::$stream->get( 24 );
				$data = Oodle::decode( $data );

				if( $data === false ) { 

					return ErrorHandler::error( "Oшибка распаковки oodle cжатых данных" );

				} 

				break;

			default : 

			return ErrorHandler::error( "Неизвестный метод сжатия '{$flags[3]}' в ddsx файле" );   

		}

		switch( $format ) {

			case "\x32\x00\x00\x00" :

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x8 + 0x1000 + 0x20000 * ( $mipmaps > 1 );
				$header_dwPitchOrLinearSize = (int)( ( $width * $bitsperpixel + 7 ) / 8 );
				$mainImageSize = $width * $height;

				$pixelformat_dwFlags = 0x40;
				$pixelformat_dwFourCC = 0;
				$pixelformat_dwRGBBitCount = $bitsperpixel;
				$pixelformat_dwRBitMask = 0x000000FF;
				$pixelformat_dwGBitMask = 0x000000FF;
				$pixelformat_dwBBitMask = 0x000000FF;
				$pixelformat_dwABitMask = 0x00000000;

				break;

			case "\x33\x00\x00\x00" : 

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x8 + 0x1000 + 0x20000 * ( $mipmaps > 1 );
				$header_dwPitchOrLinearSize = (int)( ( $width * $bitsperpixel + 7 ) / 8 );
				$mainImageSize = $width * $height * 16;

				$pixelformat_dwFlags = 0x40;
				$pixelformat_dwFourCC = 0;
				$pixelformat_dwRGBBitCount = $bitsperpixel;
				$pixelformat_dwRBitMask = 0x0000F800;
				$pixelformat_dwGBitMask = 0x000007E0;
				$pixelformat_dwBBitMask = 0x0000001F;
				$pixelformat_dwABitMask = 0x00000000;

				break;

			case "\x15\x00\x00\x00" :

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x8 + 0x1000 + 0x20000 * ( $mipmaps > 1 );
				$header_dwPitchOrLinearSize = (int)( ( $width * $bitsperpixel + 7 ) / 8 );
				$mainImageSize = $width * $height * 32;

				$pixelformat_dwFlags = 0x1 + 0x40;
				$pixelformat_dwFourCC = 0;
				$pixelformat_dwRGBBitCount = $bitsperpixel;
				$pixelformat_dwRBitMask = 0x00FF0000;
				$pixelformat_dwGBitMask = 0x0000FF00;
				$pixelformat_dwBBitMask = 0x000000FF;
				$pixelformat_dwABitMask = 0xFF000000;

				break;

			case "DXT1" : 	

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x1000 + 0x20000 * ( $mipmaps > 1 ) + 0x80000;
				$header_dwPitchOrLinearSize = ( $width * $height ) / 2;
				$mainImageSize = $header_dwPitchOrLinearSize;

				$pixelformat_dwFlags = 0x4;
				$pixelformat_dwFourCC = 0x31545844;
				$pixelformat_dwRGBBitCount = 0;
				$pixelformat_dwRBitMask = 0;
				$pixelformat_dwGBitMask = 0;
				$pixelformat_dwBBitMask = 0;
				$pixelformat_dwABitMask = 0;

				break;

			case "DXT5" :  

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x1000 + 0x20000 * ( $mipmaps > 1 ) + 0x80000;
				$header_dwPitchOrLinearSize = $width * $height;
				$mainImageSize = $header_dwPitchOrLinearSize;

				$pixelformat_dwFlags = 0x4;
				$pixelformat_dwFourCC = 0x35545844;
				$pixelformat_dwRGBBitCount = 0;
				$pixelformat_dwRBitMask = 0;
				$pixelformat_dwGBitMask = 0;
				$pixelformat_dwBBitMask = 0;
				$pixelformat_dwABitMask = 0;

				break;

			case "BC7 " :  

				$header_dwFlags = 0x1 + 0x2 + 0x4 + 0x1000 + 0x20000 * ( $mipmaps > 1 ) + 0x80000;
				$header_dwPitchOrLinearSize = $width * $height;
				$header_dwDepth = 1;
				$mainImageSize = $header_dwPitchOrLinearSize;

				$pixelformat_dwFlags = 0x4;
				$pixelformat_dwFourCC = 0x30315844; //DX10
				$pixelformat_dwRGBBitCount = 0;
				$pixelformat_dwRBitMask = 0;
				$pixelformat_dwGBitMask = 0;
				$pixelformat_dwBBitMask = 0;
				$pixelformat_dwABitMask = 0;

				break;  

			case "ATI2" :
				break;

			default:

				file_put_contents( $ddsfile, $data );
				return ErrorHandler::error( "Неизвестный формат '{$format}' ddsx файла" );   

		}

		if( $flags[2] & 0x04 ){ 

			$ddsData = substr( $data, -$mainImageSize );

		}
		else {

			$ddsData = substr( $data, 0, $mainImageSize );	

		}

		if( isset($pixelformat_dwFlags) ) {
			$ddsHeader = $header_magic; // magic
			$ddsHeader .= pack( 'L', $header_dwSize ); // dwSize                            
			$ddsHeader .= pack( 'L', $header_dwFlags ); // dwFlags
			$ddsHeader .= pack( 'L', $header_dwHeight ); // dwHeight
			$ddsHeader .= pack( 'L', $header_dwWidth ); // dwWidth
			$ddsHeader .= pack( 'L', $header_dwPitchOrLinearSize ); // dwPitchOrLinearSize
			$ddsHeader .= pack( 'L', $header_dwDepth ); // dwDepth
			$ddsHeader .= pack( 'L', $header_dwMipMapCount ); // dwMipMapCount
			$ddsHeader .= pack( 'L11', 0,0,0,0,0,0,0,0,0,0,0 ); // dwReserved 

			$pixelformat = pack( 'L', $pixelformat_dwSize ); // DDS_PIXELFORMAT dwSize 
			$pixelformat .= pack( 'L', $pixelformat_dwFlags ); // DDS_PIXELFORMAT dwFlags 
			$pixelformat .= pack( 'L', $pixelformat_dwFourCC ); // DDS_PIXELFORMAT dwFourCC 
			$pixelformat .= pack( 'L', $pixelformat_dwRGBBitCount ); // DDS_PIXELFORMAT dwRGBBitCount  
			$pixelformat .= pack( 'L', $pixelformat_dwRBitMask ); // DDS_PIXELFORMAT dwRBitMask 
			$pixelformat .= pack( 'L', $pixelformat_dwGBitMask ); // DDS_PIXELFORMAT dwGBitMask 
			$pixelformat .= pack( 'L', $pixelformat_dwBBitMask ); // DDS_PIXELFORMAT dwBBitMask 
			$pixelformat .= pack( 'L', $pixelformat_dwABitMask ); // DDS_PIXELFORMAT dwABitMask 
			$ddsHeader .= $pixelformat;

			$ddsHeader .= pack( 'L', $header_dwCaps ); // dwCaps
			$ddsHeader .= pack( 'L', 0 ); // dwCaps2
			$ddsHeader .= pack( 'L', 0 ); // dwCaps3
			$ddsHeader .= pack( 'L', 0 ); // dwCaps4
			$ddsHeader .= pack( 'L', 0 ); // dwReserved2

			// заголовок DDS_HEADER_DXT10 для DX10
			if( $pixelformat_dwFourCC === 0x30315844 ){

				$ddsHeader .= "\x62\x00\x00\x00\x03\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00"; 

			}

			$dds = $ddsHeader . $ddsData;   

			if( file_put_contents( $ddsfile, $dds ) === false ) {

				return ErrorHandler::error( "Не удалось записать файл .dds" );

			};

		}

		return true;

	}    

}
?>