<?php

class VromfsBin {
	
	const MAGIC = "VRFs\x00\x00PC";

	private $stream;
	private $fat;
	
	function __construct(){

		$this->stream = new DataStream;

	}

	public function open( $filename ){

		if( !$this->stream->open( $filename ) ){

			return ErrorHandler::error( "Не удалось открыть файл '$filename'" );

		};

		$signature = $this->stream->string( 8 );

		if( $signature !== self::MAGIC ){

			return ErrorHandler::error( "Неверный тип файла '$filename'" );

		}

		$dataSize = $this->stream->uint32();
		$packedSize = $this->stream->uint24();
		$packType = $this->stream->byte();

		// незапакованные данные
		if( $packedSize === 0 ){

			$data = $this->stream->get( 0x10 );

		}
		else if( $packType === 0xC0 ) {

			$data = $this->decodeZstd( $packedSize );
			
		}
		else if( $packType === 0x40 ) {

			$data = $this->decodeZstd( $packedSize );

		}
		else {

			return ErrorHandler::error( "Неизвесный тип сжатия {$packType}" );

		}
		
		$this->stream->set( $data ); 

		$this->readFat();

		return $this->fat;

	}
	
	public function loadFile( $filename ){
		
		$filename = FileName::normalizeFileName( $filename );     
				
		$data = false;
		
		if( isset( $this->fat[$filename] ) ){
			
			$fileInfo = $this->fat[$filename];
			
			$data = $this->stream->get( $fileInfo['offset'], $fileInfo['size'] );
			
		}
		
		return $data;
		
	}
	
	public function extractFile( $filename, $dest ){
		
		$filename = FileName::normalizeFileName( $filename );	
			
		$data = false;
		
		if( isset( $this->fat[$filename] ) ){
			
			$fileInfo = $this->fat[$filename];
			
			$data = $this->stream->get( $fileInfo['offset'], $fileInfo['size'] );
			
			file_put_contents( $dest, $data );
			
		}
		
		return !!$data;
		
	}
	
	public function expand( $path ){
		
		$path = FileName::normalizeDirName( $path );
		
		foreach( $this->fat as $fileName => $fileInfo ){
			
			$data = $this->stream->get( $fileInfo['offset'], $fileInfo['size'] );
			
			$dstFileName =  $path . $fileName;
			$dstPath = dirname( $dstFileName );
			
			if( !is_dir( $dstPath ) ) {
				
				mkdir( $dstPath, 0777, true );
			
			}
			
			file_put_contents( $dstFileName, $data );
			
		}
		
		return true;
		
	}

	private function decodeZstd( $packedSize ){

		$packed = $this->stream->get( 16, $packedSize );

		$key = "\x55\xAA\x55\xAA\x0F\xF0\x0F\xF0\x55\xAA\x55\xAA\x48\x12\x48\x12";

		for( $i = 0; $i < 16; $i++ ){

			$packed[$i] = chr( ord( $packed[$i] ) ^ ord( $key[$i] ) );

		}

		$pos = 4 * (int) ( ($packedSize - 16) / 4 );
		$key = "\x48\x12\x48\x12\x55\xAA\x55\xAA\x0F\xF0\x0F\xF0\x55\xAA\x55\xAA";

		for( $i = 0; $i < 16; $i++ ){

			$packed[$i+$pos] = chr( ord( $packed[$i+$pos] ) ^ ord( $key[$i] ) );

		}

		return Zstd::decode( $packed );

	}

	private function readFat() {

		$fileNames = $this->readFileNames( $this->stream );
		$fileInfos = $this->readFileInfos( $this->stream );

		$this->fat = array_combine( $fileNames, $fileInfos );

	}

	private function readFileNames(){

		$this->stream->rewind();
		$namesOffset = $this->stream->uint32();
		$namesCount = $this->stream->uint32();
		
		$this->stream->seek( $namesOffset );
		$fileNames = array();

		for( $i = 0; $i < $namesCount; $i++ ){

			$nameOffset = $this->stream->uint32();
			$this->stream->forward( 0x04 );

			$this->stream->push();
			$this->stream->seek( $nameOffset );
			$fileNames[$i] = FileName::normalizeFileName( $this->stream->cstring() );
			$this->stream->pop();

		}

		return $fileNames;

	}

	private function readFileInfos(){

		$this->stream->seek( 0x10 );
		$infoOffset = $this->stream->uint32();
		$filesCount = $this->stream->uint32();

		$this->stream->seek( $infoOffset );
		$fileInfos = array();

		for( $i = 0; $i < $filesCount; $i++ ){

			$fileInfos[$i] = array(
				'offset' => $this->stream->uint32(),
				'size' => $this->stream->uint32()
			);

			$this->stream->forward( 0x08 );

		}

		return $fileInfos;

	}

}
?>
