<?php

class DataStream {

	private $_stream;
	private $_type;
	private $_stack;

	// открываем файловый поток
	public function open( $filename ){

		if( is_resource( $this->_stream ) ){

			fclose( $this->_stream );

		}

		$this->_type = 'file';
		$this->_stream = fopen( $filename, 'r' );

		$this->_stack = array();

		return !!$this->_stream;

	}

	// открываем поток в памяти и записываем туда данные
	public function set( $data = "" ){

		if( is_resource( $this->_stream ) ){

			fclose( $this->_stream );

		}

		$this->_type = 'memory';
		$this->_stream = fopen( 'php://memory', 'r+' );
		fwrite( $this->_stream, $data );		
		rewind( $this->_stream );

		$this->_stack = array();

		return !!$this->_stream;

	}

	// возвращает полное содержимое 
	public function get( $from = 0, $length = 0 ){
		
		$string = "";

		fseek( $this->_stream, $from );

		if( $length === 0 )  {

			$length = $this->size() - $from;

		}

		if( $length )  {

			$string = fread( $this->_stream, $length );

		}

		return $string;

	}

	public function close(){

		if( $this->_stream ){

			fclose( $this->_stream );

		}

	}

	// размер файла
	public function size() { 

		$stat = fstat( $this->_stream );

		return $stat ? $stat['size'] : false;

	}

	// конец файла?
	public function eof(){

		return $this->position() >= $this->size();

	}

	// выравнивание до $size, по умолчанияю DWORD
	public function align( $size = 4 ) {

		$offset = ftell ( $this->_stream );    	
		$align = $offset % $size;

		if( $align !== 0 ) {      

			$align = fseek( $this->_stream, $size - $align, SEEK_CUR );  

		}

		return ( $align === 0 );

	}

	// позиционирование
	public function seek( $offset ){

		return ( fseek( $this->_stream, $offset ) === 0 );

	}

	public function forward( $offset ){

		return ( fseek( $this->_stream, $offset, SEEK_CUR ) === 0 );

	} 

	public function backward( $offset ){

		return ( fseek( $this->_stream, -$offset, SEEK_CUR ) === 0 );

	} 

	public function rewind(){

		return rewind( $this->_stream );

	}

	public function end( $offset = 0 ){

		return ( fseek( $this->_stream, -$offset, SEEK_END ) === 0 );

	}

	// текущая позиция в потоке
	public function position(){

		return ftell( $this->_stream );

	} 

	// запоминаем позицию
	public function push(){

		array_push( $this->_stack, $this->position() );

	} 

	// восстанавливаем запомненную позицию
	public function pop(){

		$this->seek( array_pop( $this->_stack ) );

	} 
	
	/**
	* булевый тип
	*/
	public function bool(){  

		return !!unpack( 'C', fread( $this->_stream, 1 ) )[1];

	}

	/**
	* 8-и битные целые
	*/
	// unsigned char
	public function byte(){  

		return unpack( 'C', fread( $this->_stream, 1 ) )[1];

	}
	public function uint8(){  

		return unpack( 'C', fread( $this->_stream, 1 ) )[1];

	}
	// signed char
	public function int8(){

		return unpack( 'c', fread( $this->_stream, 1 ) )[1];

	}

	/**
	*  16-х битные целые значения
	*/

	// unsigned short
	public function uint16(){

		return unpack( 'S', fread( $this->_stream, 2 ) )[1];           

	}    

	// unsigned short vector
	public function wvector( $num ){

		return array_values( unpack( "S{$num}", fread( $this->_stream, 2 * $num ) ) );        

	}

	// unsigned short littleendian 
	public function uint16le(){

		return unpack( 'v', fread( $this->_stream, 2 ) )[1];           

	}

	// unsigned short bigendian 
	public function uint16be(){

		return unpack( 'n', fread( $this->_stream, 2 ) )[1];           

	}
	
	// signed short
	public function int16(){

		return unpack( 's', fread( $this->_stream, 2 ) )[1];           

	}  

	/**
	*  24-х битные целые значения
	*/

	// unsigned int
	public function uint24(){

		return unpack( 'S', fread( $this->_stream, 2 ) )[1] + 65536 * unpack( 'C', fread( $this->_stream, 1 ) )[1];           

	}

	/**
	*  32-х битные целые значения
	*/

	// unsigned int
	public function uint32(){

		return unpack( 'L', fread( $this->_stream, 4 ) )[1];           

	}

	// unsigned int vector
	public function uvector( $num ){

		return array_values( unpack( "L{$num}", fread( $this->_stream, 4 * $num ) ) );        

	}

	// unsigned int littleendian 
	public function uint32le(){

		return unpack( 'V', fread( $this->_stream, 4 ) )[1];           

	}

	// unsigned int bigendian 
	public function uint32be(){

		return unpack( 'N', fread( $this->_stream, 4 ) )[1];           

	}

	// signed int
	public function int32(){

		return unpack( 'l', fread( $this->_stream, 4 ) )[1];           

	}

	// signed int vector
	public function ivector( $num ){

		return array_values( unpack( "l{$num}", fread( $this->_stream, 4 * $num ) ) );        

	}



	/**
	*  64-х битные целые значения
	*/

	// unsigned long
	public function uint64(){

		return unpack( 'Q', fread( $this->_stream, 8 ) )[1];           

	}

	// unsigned long littleendian 
	public function uint64le(){

		return unpack( 'P', fread( $this->_stream, 8 ) )[1];           

	}

	// unsigned long bigendian
	public function uint64be(){

		return unpack( 'J', fread( $this->_stream, 8 ) )[1];           

	}

	/**
	* 16-х битные с плавающей точкой (половинной точности, half float)
	*/
	function half() {

		$in = $this->uint16();
		$t1 = $in & 0x7fff; // Non-sign bits
		$t2 = $in & 0x8000; // Sign bit
		$t3 = $in & 0x7c00; // Exponent
		$t1 <<= 13; // Align mantissa on MSB
		$t2 <<= 16; // Shift sign bit into position

		$t1 += 0x38000000; // Adjust bias

		$t1 = ( $t3 == 0 ? 0 : $t1 ); // Denormals-as-zero

		$t1 |= $t2; // Re-insert sign bit

		return unpack( "f", pack( "L", $t1 ) )[1];

	} 

	/**
	* 32-х битные с плавающей точкой
	*/

	// float 
	public function float(){

		return unpack( 'f', fread( $this->_stream, 4 ) )[1];           

	}

	// float littleendian 
	public function float_le(){

		return unpack( 'g', fread( $this->_stream, 4 ) )[1];           

	}

	// float bigendian
	public function float_be(){

		return unpack( 'G', fread( $this->_stream, 4 ) )[1];           

	}

	/**
	* 64-х битные с плавающей точкой
	*/

	// double 
	public function double(){

		return unpack( 'f', fread( $this->_stream, 8 ) )[1];           

	}

	// double littleendian 
	public function double_le(){

		return unpack( 'g', fread( $this->_stream, 8 ) )[1];           

	}

	// double bigendian
	public function double_be(){

		return unpack( 'G', fread( $this->_stream, 8 ) )[1];           

	}

	// float vector
	public function fvector( $num ){

		return array_values( unpack( "f{$num}", fread( $this->_stream, 4 * $num ) ) );  
	}

	// float matrix [rows x columns]
	public function fmatrix( $rows = 4, $columns = 4 ){

		$res = array();

		for( $r = 0; $r < $rows; $r++ ){

			$res[$r] = $this->fvector( $columns );

		}

		return $res;

	}


	/**
	* Строки
	*/

	public function string( $size ){

		return $size ? fread( $this->_stream, $size ) : "";

	}

	public function cstring( $maxSize = 4096 ){

		$position = 0;
		$string = "";     	

		do {

			$symbol = fread( $this->_stream, 1 ); 	

			if( $symbol === "\0" ) { break; }

			$string .= $symbol;

		} while ( ++$position < $maxSize );

		return $string;

	}

	public function hexstring( $size ){

		$res = '';

		while( $size-- > 0 ){

			$res .=  sprintf( "%02X",  $this->byte() );

		}  

		return $res;

	}
	
	// поиск строки
	public function search( $string ){

		$currentPosition = $this->position();		
		$isFound = false;
		$stringSize = strlen( $string );
		
		do{
			
			$tmp = fread( $this->_stream, $stringSize );
			
			if( $tmp == $string  ){
				
				$isFound = true;
				fseek( $this->_stream, -$stringSize, SEEK_CUR );
				break;
				
			}
			
			fseek( $this->_stream, -( $stringSize-1 ), SEEK_CUR );
			
		} while ( !feof( $this->_stream ) );
		
        if( !$isFound ){
        	
            $this->seek( $currentPosition );
              
        }
        
        return $isFound;
    }

}
?>
