<?php

/**
*  Класс для работы с алгоритмом сжатия LZ4
* 
* Зависит от Temp, FileBuffer и lz4x.exe
* 
*/

define ( "LZ4_LEGACY_MAGIC", "\x02\x21\x4C\x18" );

class Lz4 {

    private static $_error;

    /**
    * Распаковывка "сырых" данных, без заголовка
    * 
    * @param mixed $string
    */
    public static function decodeRaw( $string ){  
        
        $uncompressedFile = Temp::getFileName();  
        $compressedFile = $uncompressedFile . ".lz4";  
        
        $lz4Header = LZ4_LEGACY_MAGIC . pack( 'L', strlen( $string ) );

        if( false === @file_put_contents( $compressedFile, $lz4Header . $string ) ){

            return self::_error( "Не удалось записать временный файл" );

        };
        
        $exe = __DIR__ . "\\bin\\lz4x.exe";
        $cmd = "\"{$exe}\" -d -f \"$compressedFile\" \"$uncompressedFile\"";

        if( exec( $cmd, $output, $code ) === false ){

        	@unlink( $compressedFile );
            return self::_error( "Не удалось запустить декомпрессор" );    

        };
        
        @unlink( $compressedFile );

        if( $code ){

            return self::_error( "Ошибка декомпрессии" );

        }

        $string = @file_get_contents( $uncompressedFile );
        @unlink( $uncompressedFile );

        if( $string === false ) {

            return self::_error( "Не удалось прочитать распакованные данные" );

        }

        return $string;

    }

    private static function _error( $error ){

        self::$_error = $error;

        return false;

    }

    public static function getLastError(){

        return self::$_error;

    }
}

?>