<?php

class ErrorHandler {

	protected static $errors = array();

	public static function error( $message, $data = NULL ){

		$error = self::trace();

		$error['kind'] = 'error';
		$error['message'] = $message;
		$error['data'] = $data;

		self::$errors[] = $error;

		return NULL;

	}

	public static function fatal( $message, $data = NULL){

		$error = self::trace();   

		self::printErrors();
		echo "<p>Неустранимая ошибка в cтроке {$error['line']} файла '{$error['file']}'<br>";
		echo "{$message}</p>";  

		if( $data ){

			echo "<pre>";				
			echo print_r( $data, true );
			echo "</pre>";
			
		}

		die();

	}   
	
	public static function hasErrors(){
		
		return !!( count( self::$errors ) );
		
	}

	public static function printErrors(){

		$html = "";

		foreach( self::$errors as $error ){

			$html .= "<p>";

			if( isset( $error['function'] ) ){

				if( isset( $error['class'] ) ) {

					$html .= "Ошибка в методе {$error['class']}{$error['type']}{$error['function']}()<br>";
					$html .=  "Строка {$error['line']} файл '{$error['file']}'<br>";

				}
				else {

					$html .= "Ошибка в функции {$error['function']}()<br>";
					$html .= "Строка {$error['line']} файл '{$error['file']}'<br>";

				}

			}
			else {

				$html .=  "Ошибка в строке {$error['line']} файла '{$error['file']}<br>";

			}


			$html .= "{$error['message']}";

			if( $error['data'] ){

				$html .= "<pre>";				
				$html .= print_r( $error['data'], true );
				$html .= "</pre>";
			}

			$html .= "</p>";

		}

		// print errors
		echo $html;
		
		// clear errors		
		self::$errors = array();
		
		return;

	}



	private static function trace(){

		$stack = debug_backtrace( 3 );

		$error = array();

		// строка и файл
		$error['file'] = $stack[1]['file'];
		$error['line'] = $stack[1]['line'];  

		// ошибка в методе?
		if( isset( $stack[2] ) ) {

			if( isset( $stack[2]['class'] ) )     $error['class'] = $stack[2]['class'];
			if( isset( $stack[2]['type'] ) )      $error['type'] = $stack[2]['type'];
			if( isset( $stack[2]['function'] ) )  $error['function'] = $stack[2]['function'];

		}

		return $error;

	}



}
?>
