<?php

require_once "baseclass.php";
require_once "ErrorHandler/ErrorHandler.php";
require_once "FileName/FileName.php";
require_once "Temp/Temp.php";

define( 'WINDOWS', "Windows" );
define( 'UNIX', "Unix" );

/**
*
* @staticvar boolean $echo
*/
function _log() {

	static $echo = true;

	$args = func_get_args();

	if( count( $args ) == 0 ) {

		$echo = !$echo;

	}

	if( $echo ) {

		foreach( $args as $value ) {

			if( is_array( $value ) ) {

				echo "<pre>";
				print_r( $value );
				echo "</pre>";

			}
			else {

				echo "<pre>{$value}</pre>";

			}

		}

	}

}

/**
* Создание папки
*
* @param string $path Путь к папке
* @param int $mode Права доступа
*/
function createDir( $path, $mode = 0777 ) {

	if( !file_exists( $path )
	|| !is_dir( $path ) ) {

		mkdir( $path, $mode, true );

	}

	return $path;

}

/**
* Очистка папки
*
* @param string $path Путь к папке
*/
function clearDir( $path ) {

	if( file_exists( $path )
	&& is_dir( $path ) ) {

		removeDir( $path );

	}

	createDir( $path );

	return true;

}

/**
* Удаление папки
*
* @param string $dir  Путь к папке
*/
function removeDir( $dir ) {

	if( is_dir( $dir ) ) {

		$objects = scandir( $dir );

		foreach( $objects as $object ) {

			if( $object !== "."
			&& $object !== ".." ) {

				if( filetype( $dir . "/" . $object ) == "dir" ) {

					removeDir( $dir . "/" . $object );
				}
				else {

					unlink( $dir . "/" . $object );
				}
			}
		}

		reset( $objects );
		rmdir( $dir );
	}

	return true;

}

/**
*
* @param type $domain
* @param type $port
* @param type $timeout
* @return type
*/
function ping( $domain, $port = 80, $timeout = 10 ) {
	$domain = trim( str_replace( array( "http://", "https://" ), "", $domain ), "/" );
	$starttime = microtime( true );
	$file = @fsockopen( $domain, $port, $errno, $errstr, $timeout );
	$stoptime = microtime( true );
	$status = 0;

	if( !$file )
		$status = -1;  // Site is down
	else {
		fclose( $file );
		$status = ( $stoptime - $starttime ) * 1000;
		$status = floor( $status );
	}
	return $status;

}

function post_request( $url, $query ) {

	$request = curl_init();
	curl_setopt_array( $request, array(
		CURLOPT_URL => $url,
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_POST => true,
		CURLOPT_POSTFIELDS => $query,
		CURLOPT_SSL_VERIFYPEER => false,
	) );
	$response = curl_exec( $request );

	if( curl_errno( $request ) ) {

		echo 'Curl error: ' . curl_error( $request );
	}

	curl_close( $request );

	return $response;

}

function get_request( $url ) {

	$request = curl_init();
	curl_setopt_array( $request, array(
		CURLOPT_URL => $url,
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_POST => false,
		CURLOPT_FAILONERROR => true,
		CURLOPT_SSL_VERIFYPEER => false,
		CURLOPT_FOLLOWLOCATION => true,
	) );
	$response = curl_exec( $request );

	if( curl_errno( $request ) ) {

		echo 'Curl error: ' . curl_error( $request );
	}

	curl_close( $request );

	return $response;

}

/**
*
* @param type $value
* @return type
*/
function sign( $value ) {

	return ( $value < 0.0 ) ? -1 : 1;

}

function _round( $value, $prec = 3 ) {
	if( is_numeric( $value ) ) {
		$v = abs( $value );
		if( $v > 100 ) {
			$value = round( $value );
		}
		else if( $v > 10 ) {
			$value = round( $value, 1 );
		}
		else if( $v > 1 ) {
			$value = round( $value, 2 );
		}
		else if( $v > 0.1 ) {
			$value = round( $value, 3 );
		}
		else if( $v > 0.01 ) {
			$value = round( $value, 4 );
		}
		else if( $v > 0.001 ) {
			$value = round( $value, 5 );
		}
		else {
			$value = round( $value, 6 );
		}
	}
	return $value;

}

// хитрая сортировка массива по ключу - сначала значения, потом массивы
/**
* Хитрая сортировка (многомерного) массива по ключу - сначала значения, потом массивы (либо наоборот)
*
* @param mixed $array Исходный массив
* @param mixed $arraysfirst Порядок сортировки. true - массивы впереди, false массивы в конце. По умолчанию false
*
* @return отсортированный массив
*/
function ksort_array_recursively( $array, $arraysfirst = false ) {
	$result = array();
	$result[0] = array();
	$result[1] = array();
	ksort( $array );
	foreach( $array as $key => $value ) {
		if( is_array( $value ) ) {
			$result['1'][$key] = ksort_array_recursively( $value, $arraysfirst );
		}
		else {
			$result['0'][$key] = $value;
		}
	}
	if( $arraysfirst ) {
		return array_merge( $result[1], $result[0] );
	}
	return array_merge( $result[0], $result[1] );

}

// Поиск в многомерном массиве
function in_array_rc( $needle, $haystack, $strict = false ) {
	foreach( $haystack as $item ) {
		if( ($strict ? $item === $needle : $item == $needle) || (is_array( $item ) && in_array_rc( $needle, $item, $strict )) ) {
			return true;
		}
	}

	return false;

}

// HEX string
function hexstring( $value, $size = 4 ) {

	$res = sprintf( "%0{$size}Xh", $value );

	if( strlen( $res ) > 5 ) {

		//    $res = substr_replace( $res, ":", -5, 0 );
	}

	return $res;

}

function translit( $str ) {

	$rus = array( 'А', 'Б', 'В', 'Г', 'Д', 'Е', 'Ё', 'Ж', 'З', 'И', 'Й', 'К', 'Л', 'М', 'Н', 'О', 'П', 'Р', 'С', 'Т', 'У', 'Ф', 'Х', 'Ц', 'Ч', 'Ш', 'Щ', 'Ъ', 'Ы', 'Ь', 'Э', 'Ю', 'Я', 'а', 'б', 'в', 'г', 'д', 'е', 'ё', 'ж', 'з', 'и', 'й', 'к', 'л', 'м', 'н', 'о', 'п', 'р', 'с', 'т', 'у', 'ф', 'х', 'ц', 'ч', 'ш', 'щ', 'ъ', 'ы', 'ь', 'э', 'ю', 'я' );
	$lat = array( 'A', 'B', 'V', 'G', 'D', 'E', 'E', 'Gh', 'Z', 'I', 'Y', 'K', 'L', 'M', 'N', 'O', 'P', 'R', 'S', 'T', 'U', 'F', 'H', 'C', 'Ch', 'Sh', 'Sch', 'Y', 'Y', 'Y', 'E', 'Yu', 'Ya', 'a', 'b', 'v', 'g', 'd', 'e', 'e', 'gh', 'z', 'i', 'y', 'k', 'l', 'm', 'n', 'o', 'p', 'r', 's', 't', 'u', 'f', 'h', 'c', 'ch', 'sh', 'sch', 'y', 'y', 'y', 'e', 'yu', 'ya' );

	return str_replace( $rus, $lat, $str );

}

// рекурсивный glob
function glob_recursive( $pattern, $flags = 0 ) {
	$files = glob( $pattern, $flags );
	foreach( glob( dirname( $pattern ) . '/*', GLOB_ONLYDIR | GLOB_NOSORT ) as $dir ) {
		$files = array_merge( $files, glob_recursive( $dir . '/' . basename( $pattern ), $flags ) );
	}
	return $files;

}

function binstr( $value ) {

	$result = "";
	$value = sprintf( "%08X", $value );

	for( $i = 0; $i < 8; $i++ ) {

		$sym = hexdec( $value[$i] );

		for( $n = 0; $n < 4; $n++ ) {

			$sym = $sym << 1;
			$bit = $sym & 0x10;
			$sym = $sym & 0x0F;

			$result .= ( $bit ? '1' : '0' );
		}
	}

	return $result;

}

?>